# Copyright (C) 2017  Lars Wirzenius
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import ick2


class ProjectAPI(ick2.ResourceApiBase):

    def __init__(self, state):
        super().__init__('projects', state)

    def get_resource_name(self, resource):
        return resource['project']

    def get_routes(self, path):  # pragma: no cover
        return super().get_routes(path) + self.get_pipeline_routes(path)

    def get_pipeline_routes(self, path):  # pragma: no cover
        pipeline_path = '{}/<project>/pipelines/<pipeline>'.format(path)
        return [
            {
                'method': 'GET',
                'path': pipeline_path,
                'callback': self.GET(self.get_pipeline),
            },
            {
                'method': 'PUT',
                'path': pipeline_path,
                'callback': self.PUT(self.set_pipeline_callback),
            },
            {
                'needs-authorization': False,
                'method': 'GET',
                'path': pipeline_path + '/+trigger',
                'callback': self.GET(self.trigger_pipeline),
            },
        ]

    def get_pipeline(self, project, pipeline):
        p = self._state.get_resource(self._type_name, project)
        for pl in p['pipelines']:
            if pl['name'] == pipeline:
                return {
                    'status': pl.get('status', 'idle'),
                }
        raise ick2.NotFound()

    def set_pipeline_callback(
            self, body, project, pipeline):  # pragma: no cover
        return self.set_pipeline(body['status'], project, pipeline)

    def set_pipeline(self, state, project, pipeline):
        allowed_changes = {
            'idle': 'triggered',
            'triggered': 'building',
            'building': 'idle',
        }
        p = self._state.get_resource(self._type_name, project)
        for pl in p['pipelines']:
            if pl['name'] == pipeline:
                old_state = pl.get('status', 'idle')
                if allowed_changes[old_state] != state:
                    raise ick2.WrongPipelineStatus(state)
                pl['status'] = state
                self._state.update_resource(self._type_name, project, p)
                return {'status': state}
        raise ick2.NotFound()

    # This needs to go away as it is not protected. Once an IDP is
    # added.
    def trigger_pipeline(self, project, pipeline):  # pragma: no cover
        return self.set_pipeline('triggered', project, pipeline)
