# Copyright 2017 Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import errno
import json
import os
import random
import socket
import sys
import time

import cliapp
import requests

from yarnutils import *


srcdir = os.environ['SRCDIR']
datadir = os.environ['DATADIR']
vars = Variables(datadir)


def random_free_port():
    MAX = 1000
    for i in range(MAX):
        port = random.randint(1025, 2**15-1)
        s = socket.socket()
        try:
            s.bind(('0.0.0.0', port))
        except OSError as e:
            if e.errno == errno.EADDRINUSE:
                continue
            print('cannot find a random free port')
            raise
        s.close()
        break
    print('picked port', port)
    return port


def wait_for_port(port):
    MAX = 5
    t = time.time()
    while time.time() < t + MAX:
        try:
            s = socket.socket()
            s.connect(('127.0.0.1', port))
        except OSError as e:
            raise
        else:
            return

def unescape(s):
    t = ''
    while s:
        if s.startswith('\\n'):
            t += '\n'
            s = s[2:]
        else:
            t += s[0]
            s = s[1:]
    return t

def write(filename, data):
    with open(filename, 'w') as f:
        f.write(data)


def cat(filename):
    MAX_CAT_WAIT = 5  # in seconds
    t = time.time()
    while time.time() < t + MAX_CAT_WAIT:
        if os.path.exists(filename):
            return open(filename, 'r').read()


def store_token(user, token):
    filename = '{}.jwt'.format(user)
    write(filename, token)


def get_token(user):
    filename = '{}.jwt'.format(user)
    return cat(filename)


def get(url, token):
    headers = {
        'Authorization': 'Bearer {}'.format(token),
    }
    r = requests.get(url, headers=headers, verify=False)
    return r.status_code, r.headers['Content-Type'], dict(r.headers), r.text


def post(url, body_text, token):
    headers = {
        'Authorization': 'Bearer {}'.format(token),
        'Content-Type': 'application/json',
    }
    r = requests.post(url, headers=headers, data=body_text, verify=False)
    return r.status_code, r.headers['Content-Type'], dict(r.headers), r.text


def put(url, body_text, token):
    headers = {
        'Authorization': 'Bearer {}'.format(token),
        'Content-Type': 'application/json',
    }
    r = requests.put(url, headers=headers, data=body_text, verify=False)
    return r.status_code, r.headers['Content-Type'], dict(r.headers), r.text


def delete(url, token):
    headers = {
        'Authorization': 'Bearer {}'.format(token),
    }
    r = requests.delete(url, headers=headers, verify=False)
    return r.status_code, r.headers['Content-Type'], dict(r.headers), r.text
